//======================================================================
//
//                    mike-codes.h
//
// Purpose:
//          - file contains all function declarations for Mikes original routines,
//            that are used within the various drivers
//
// Note:
//          - contains forward declarations for routines 
//            that were originally defined (and declared in testlinadj.cpp)
//          - ADdouble has been replaced by simple double everywhere
//
//======================================================================


/* Monte Carlo LIBOR path calculation                      */
/* template enables use of AD through operator overloading */
void path_calc(const int N, const int Nmat, const double delta,
               double L[], const double lambda[], const double z[]);


/* forward pathwise sensitivity calculation of deltas
   (commented out bits are for vega calculation) */
void path_calc_d(const int N, const int Nmat, const double delta,
                 double L[], double L_d[],
                 const double lambda[], const double z[]);


/* forward path calculation storing data
   for subsequent reverse path calculation */
void path_calc_b1(const int N, const int Nmat, const double delta,
                  double L[], const double lambda[], const double z[],
                  double L2[], double L3[]);


/* reverse path calculation of deltas using stored data
   (also calculates vegas at minimal additional cost)  */
void path_calc_b2(const int N, const int Nmat, const double delta,
                  const double L[], double L_b[],
                  const double lambda[], double lambda_b[],
                  const double z[], const double L2[], const double L3[]);


/* calculate the portfolio value v */
/* template enables use of AD through operator overloading */
void portfolio(const int N, const int Nmat, const double delta,
               const int Nopt, const int maturities[],
               const double swaprates[],
               const double L[], double* v );


/* calculate the portfolio value v, and its sensitivity to L */
/* hand-code forward mode sensitivity */
void portfolio_d(const int N, const int Nmat, const double delta,
                 const int Nopt, const int maturities[],
                 const double swaprates[],
                 const double L[], const double L_d[],
                 double *v, double v_d[]);


/* calculate the portfolio value v, and its sensitivity to L */
/* hand-code reverse mode sensitivity */
void portfolio_b(const int N, const int Nmat, const double delta, 
                 const int Nopt, const int maturities[],
                 const double swaprates[],
                 const double L[], double L_b[], double* v);
