//======================================================================
//
//                    tac++-codes.h
//
// Purpose:
//          - include file, that is commonly used by our drivers.
//
// Note:
//          - contains forward declarations for routines from
//            file tac++-codes.c
//          - file also contains the forward declarations
//            for all the derivative routines as generated by TAC++
//            (so there is in fact also a dependency on the options used
//             to generate the derivative codes that are defined in the
//             Makefile)
//
//======================================================================
#ifndef TAC_DERIV_H
#define TAC_DERIV_H

const double epsmach = 1.e-20;  // default epsilon used in calls to reldiff
extern double reldiff( double a, double b, const double epsmach); // only utility routine

//------------------------------
// NOTE:
// - actually we must choose "pjacmax" equal to
//   the "N" in the main routine of vtlmdriver.cpp
// - Also, the tangent linear code, tac-code_vtl.c must
//   have been generated with option "-jacobian #pjacobian"
//------------------------------
const int pjacmax = 80;



//================================================================================
//
//          f o r w a r d   d e c l a r a t i o n s   o f   a l l   r o u t i n e s
//
//================================================================================
//--------------------------------------------------
//          f o r w a r d   r o u t i n e s
//--------------------------------------------------
extern void model(const int N, const int Nmat, const double delta, const int Nopt, const int maturities[], const double z[], const double swapraggtes[], double L[], double lambda[], double *v);

extern void model_ext(const int N, const int Nmat, const double delta, const int Nopt, const int maturities[], const double z[], const double swaprates[], double L[], double lambda[], double *v);

extern void tac_portfolio(const int N, const int Nmat, const double delta, const int Nopt, const int maturities[], const double swaprates[], double L[], double *v);

extern void tac_portfolio_ext(const int N, const int Nmat, const double delta, const int Nopt, const int maturities[], const double swaprates[], double L[], double *v);

void tac_path_calc(const int N, const int Nmat, const double delta, double L[], double lambda[], const double z[], double Lout[]);

extern void tac_path_calc_ext(const int N, const int Nmat, const double delta, double L[], double lambda[], const double z[], double Lout[]);


//--------------------------------------------------
//          a d j o i n t   r o u t i n e s
//--------------------------------------------------
extern void model_ad(const int N, const int Nmat, const double delta, const int Nopt, const int maturities[], const double z[], const double swaprates[], double L[], double L_ad[], double lambda[], double lambda_ad[], double *v, double *v_ad);

extern void model_ext_ad(const int N, const int Nmat, const double delta, const int Nopt, const int maturities[], const double z[], const double swaprates[], double L[], double L_ad[], double lambda[], double lambda_ad[], double *v, double *v_ad);

extern void tac_portfolio_ad(const int N, const int Nmat, const double delta, const int Nopt, const int maturities[], const double swaprates[], double L[], double L_ad[], double *v, double *v_ad);

extern void tac_portfolio_ext_ad(const int N, const int Nmat, const double delta, const int Nopt, const int maturities[], const double swaprates[], double L[], double L_ad[], double *v, double *v_ad);

extern void tac_path_calc_ad(const int N, const int Nmat, const double delta, double L[], double L_ad[], double lambda[], double lambda_ad[], const double z[], double Lout[], double Lout_ad[]);

extern void tac_path_calc_ext_ad(const int N, const int Nmat, const double delta, double L[], double L_ad[], double lambda[], double lambda_ad[], const double z[], double Lout[], double Lout_ad[]);


//--------------------------------------------------
//          t l m   r o u t i n e s   (scalar mode)
//--------------------------------------------------
extern void model_tl(const int N, const int Nmat, const double delta, const int Nopt, const int maturities[], const double z[], const double swaprates[], double L[], double L_tl[], double lambda[], double lambda_tl[], double *v, double *v_tl);

extern void tac_portfolio_tl(const int N, const int Nmat, const double delta, const int Nopt, const int maturities[], const double swaprates[], double L[], double L_tl[], double *v, double *v_tl);


//--------------------------------------------------
//          t l m   r o u t i n e s   (vector mode)
//--------------------------------------------------
extern void model_vtl(const int N, const int Nmat, const double delta, const int Nopt, const int maturities[], const double z[], const double swaprates[], double L[], double L_vtl[][pjacmax], double lambda[], double *v, double *v_vtl);

extern void tac_portfolio_vtl(const int N, const int Nmat, const double delta, const int Nopt, const int maturities[], const double swaprates[], double L[], double L_vtl[][pjacmax], double *v, double *v_vtl);



#endif//TAC_DERIV_H
